@@ -4,31 +4,29 @@
 Command Line Script Support Utility
 ===================================
 
-* One line argument parsing function
-* CSV reader/writer unicode support (example impl. in official document)
-* Simple stream utility with some logging
+* One line argument parsing function and decorator
+* Simple configuration loader
+* Stream utility with some logging
+* CSV reader/writer unicode support for Python 2.x (in official document)
 * Apache accesslog parser
 
 """
 
 __title__ = 'clitool'
-__version__ = '0.2.3'
+__version__ = '0.2.5'
 __author__ = 'KITAZAKI Shigeru'
 
 # Constant values.
 
-DEFAULT_ENCODING = 'utf-8'
-
 RUNNING_MODE_ENVKEY = 'PYTHON_CLITOOL_ENV'
-RUNNING_MODE_DEVELOPMENT = 'development'
-RUNNING_MODE_STAGING = 'staging'
-RUNNING_MODE_PRODUCTION = 'production'
+
+DEFAULT_ENCODING = 'utf-8'
+DEFAULT_RUNNING_MODE = 'development'
 
 PROCESSING_REPORTING_INTERVAL = 10000
 PROCESSING_SUCCESS = 'success'
 PROCESSING_SKIPPED = 'skipped'
 PROCESSING_ERROR = 'error'
-PROCESSING_FAILURE = 'failure'
 PROCESSING_TOTAL = 'total'
 PROCESSING_TIME = 'time'
 
