@@ -11,38 +11,41 @@ import multiprocessing
 import os
 import sys
 import time
+import warnings
 from collections import Counter
 
+from six import PY3
+from six.moves import map as imap
+from six.moves import filter as ifilter
+
 from clitool import (
     PROCESSING_REPORTING_INTERVAL,
     PROCESSING_SUCCESS,
     PROCESSING_SKIPPED,
     PROCESSING_ERROR,
-    PROCESSING_FAILURE,
     PROCESSING_TOTAL,
     PROCESSING_TIME
 )
 
-
-if sys.version_info.major == 3:
+if PY3:
     import csv
     import io
 
-    def csvreader3(fp, encoding):
-        return csv.reader(io.TextIOWrapper(fp.buffer, encoding))
+    def csvreader3(fp, encoding, **kwargs):
+        return csv.reader(io.TextIOWrapper(fp.buffer, encoding), **kwargs)
 
     csvreader = csvreader3
-    imap = map  # backward compat
 
 else:
-    from itertools import imap
     from ._unicodecsv import UnicodeReader
 
-    def csvreader2(fp, encoding):
-        return UnicodeReader(fp, encoding=encoding)
+    def csvreader2(fp, encoding, **kwargs):
+        return UnicodeReader(fp, encoding=encoding, **kwargs)
 
     csvreader = csvreader2
 
+warnings.simplefilter("always")
+
 
 class SimpleDictReporter(object):
     """ Reporting class for streamer API.
@@ -81,11 +84,23 @@ class RowMapper(object):
     Since this object is aimed to use with :class:`Streamer`,
     mapping is fired to call this.
 
+    If you know header names a priori, use standard `namedtuple
+    <http://docs.python.org/2/library/collections.html>`_ instead.
+    But the case that keys contains non-ascii, such as Japanese text, this
+    class may be useful.
+
     :param header: list of header values.
+    :type header: tuple
+    :param loose: loosly match flag.
+    :type loose: bool
     """
 
-    def __init__(self, header=None, *args, **kwargs):
+    ERRMSG = "Couldn't map given row: header-length=%d, row-length=%d"
+    LOOSEMSG = "Length is not matched: header-length=%d, row-length=%d"
+
+    def __init__(self, header=None, loose=False, *args, **kwargs):
         self.header = header
+        self.loose = loose
 
     def __call__(self, row, *args, **kwargs):
         """
@@ -93,49 +108,18 @@ class RowMapper(object):
         :type row: tuple or list
         :rtype: dictionary after binding with header values.
         """
+        if not row:
+            return
         if self.header is None or self.header == row:
             self.header = row
             logging.info("New header is set, because no header was given.")
             return
         if len(self.header) == len(row):
             return dict(zip(self.header, row))
-        logging.info("Couldn't map given row: header-length=%d, row-length=%d",
-            len(self.header), len(row))
-
-
-class Functions(object):
-
-    """
-    :param item: item to process.
-    :rtype: True if skipped, False if error, and valid object if processed.
-    """
-
-    def __init__(self, *args):
-        self.procedures = tuple(filter(lambda r: r, args))
-        logging.debug("%d procedures are set.", len(self.procedures))
-        self.reporting_interval = PROCESSING_REPORTING_INTERVAL
-
-    def __call__(self, tup):
-        i, item = tup
-        if i % self.reporting_interval == 0:
-            logging.info("Processing %dth item.", i)
-        if not item:  # Skip
-            return True
-        ret = item
-        for proc in self.procedures:
-            try:
-                ret = proc(ret)
-            except KeyboardInterrupt:
-                logging.info("Stopped by user interruption at %dth item.", i)
-                raise SystemExit("Stopped by user interruption.")
-            except:
-                logging.error("Fail to process at %dth item.", i)
-                return None
-            if ret is None:  # Skip
-                return True
-            elif not ret:  # Error
-                return False
-        return ret
+        if self.loose:
+            logging.info(RowMapper.LOOSEMSG, len(self.header), len(row))
+            return dict(zip(self.header, row))
+        logging.error(RowMapper.ERRMSG, len(self.header), len(row))
 
 
 # Can not recycle pool . . .
@@ -165,12 +149,15 @@ class Streamer(object):
     """
 
     def __init__(self, callback=None, *args, **kwargs):
-        self.func = Functions(*args)
+        procs = tuple(filter(lambda r: r, args))
+        logging.debug("%d procedures are set.", len(procs))
+        self.procedures = procs
         self.collect = callback or (lambda r: r)
         self.processes = kwargs.get('processes')
         if self.processes and self.processes > multiprocessing.cpu_count():
                 logging.warn("given processes is %d, count of CPU is %d" % (
                     self.processes, multiprocessing.cpu_count()))
+        self.reporting_interval = PROCESSING_REPORTING_INTERVAL
 
     def consume(self, stream, chunksize=1):
         """ Consuming given strem object and returns processing stats.
@@ -183,38 +170,53 @@ class Streamer(object):
             PROCESSING_TOTAL: 0,
             PROCESSING_SKIPPED: 0,
             PROCESSING_SUCCESS: 0,
-            PROCESSING_ERROR: 0,
-            PROCESSING_FAILURE: 0
+            PROCESSING_ERROR: 0
         }
+        rs = ifilter(lambda r: r, stream)
         if self.processes:
             pool = multiprocessing.Pool(processes=self.processes)
-            results = pool.imap_unordered(self.func,
-                        enumerate(stream, 1), chunksize=chunksize)
+            for f in self.procedures:
+                rs = pool.imap_unordered(f, ifilter(lambda r: r, rs),
+                        chunksize=chunksize)
         else:
-            results = imap(self.func, enumerate(stream, 1))
+            for f in self.procedures:
+                rs = imap(f, ifilter(lambda r: r, rs))
         start = time.time()
-        for processed in results:
-            if processed is True:
-                stats[PROCESSING_SKIPPED] += 1
-            elif processed:
-                stats[PROCESSING_SUCCESS] += 1
-                self.collect(processed)
-            elif processed is None:
-                stats[PROCESSING_FAILURE] += 1
-            else:
-                stats[PROCESSING_ERROR] += 1
-            stats[PROCESSING_TOTAL] += 1
-        if self.processes:
-            pool.close()
-            pool.join()
-        stats[PROCESSING_TIME] = time.time() - start
-        logging.info("finish processing on %f[sec].", stats[PROCESSING_TIME])
-        logging.info(
-            "STATS: total=%d, skipped=%d, success=%d, error=%d, failure=%d",
-            stats[PROCESSING_TOTAL], stats[PROCESSING_SKIPPED],
-            stats[PROCESSING_SUCCESS], stats[PROCESSING_ERROR],
-            stats[PROCESSING_FAILURE])
-        return stats
+        i = 0
+        try:
+            while 1:
+                processed = next(rs)
+                if processed is None:
+                    stats[PROCESSING_SKIPPED] += 1
+                elif processed is False:
+                    stats[PROCESSING_ERROR] += 1
+                else:
+                    stats[PROCESSING_SUCCESS] += 1
+                    self.collect(processed)
+                i += 1
+                stats[PROCESSING_TOTAL] += 1
+                if i % self.reporting_interval == 0:
+                    logging.info(" ===> Processed %dth item <=== ", i)
+        except StopIteration:
+            pass
+        except KeyboardInterrupt:
+            logging.info("Stopped by user interruption at %dth item.", i)
+            raise
+        except:
+            e = sys.exc_info()[1]
+            logging.error(e)
+            raise
+        finally:
+            if self.processes:
+                pool.close()
+                pool.join()
+            stats[PROCESSING_TIME] = time.time() - start
+            logging.info(
+                "STATS: total=%d, skipped=%d, success=%d, error=%d on %f[sec]",
+                stats[PROCESSING_TOTAL], stats[PROCESSING_SKIPPED],
+                stats[PROCESSING_SUCCESS], stats[PROCESSING_ERROR],
+                stats[PROCESSING_TIME])
+            return stats
 
 
 class CliHandler(object):
@@ -258,23 +260,22 @@ class CliHandler(object):
         :rtype: list
         """
         stats = []
-        try:
-            if files:
-                logging.info("Input file count: %d", len(files))
-                for fp in files:
-                    fname = fp.name
-                    logging.info("Start processing: %s", fname)
-                    reader = self.reader(fp, encoding)
-                    parsed = self.streamer.consume(reader, chunksize)
-                    logging.info("End processing: %s", fname)
-                    parsed['file'] = fname
-                    stats.append(parsed)
-            else:
-                reader = self.reader(sys.stdin, encoding)
-                parsed = self.streamer.consume(reader)
+        if files:
+            logging.info("Input file count: %d", len(files))
+            for fp in files:
+                fname = fp.name
+                logging.info("Start processing: %s", fname)
+                reader = self.reader(fp, encoding)
+                parsed = self.streamer.consume(reader, chunksize)
+                logging.info("End processing: %s", fname)
+                parsed['file'] = fname
                 stats.append(parsed)
-        except KeyboardInterrupt:
-            logging.info("Stopped by user interruption.")
+                if not fp.closed:
+                    fp.close()
+        else:
+            reader = self.reader(sys.stdin, encoding)
+            parsed = self.streamer.consume(reader)
+            stats.append(parsed)
         return stats
 
 
@@ -284,27 +285,9 @@ class CsvHandler(CliHandler):
         return csvreader(fp, encoding)
 
 
-def clistream(Handler, reporter, *args, **kwargs):
-    """ Handle stream data on command line interface,
-    and returns statistics of success, error, and total amount.
-
-    :param Handler: Handler for file-like streams.
-    :type Handler: CliHandler which supports `handle` method.
-    :param reporter: callback to report processed value
-    :type reporter: callable
-    :param args: functions to parse each item in the stream.
-    :param kwargs: keywords, including ``files`` and ``input_encoding``.
-    :rtype: list
-    """
-    # Follow the rule of `parse_arguments()`
-    files = kwargs.get('files')
-    encoding = kwargs.get('input_encoding')
-    processes = kwargs.get('processes')
-    chunksize = kwargs.get('chunksize')
-
-    s = Streamer(reporter, processes=processes, *args)
-    handler = Handler(s)
-
-    return handler.handle(files, encoding, chunksize)
+def clistream(reporter, *args, **kwargs):
+    warnings.warn("use 'clitool.cli.clistream' instead.", DeprecationWarning)
+    from .cli import clistream as new_clistream
+    return new_clistream(reporter, *args, **kwargs)
 
 # vim: set et ts=4 sw=4 cindent fileencoding=utf-8 :
